@@ -5,11 +5,7 @@
  * pgpool: a language independent connection pool server for PostgreSQL
  * written by Tatsuo Ishii
  *
-<<<<<<< HEAD
- * Copyright (c) 2003-2010	PgPool Global Development Group
-=======
  * Copyright (c) 2003-2013	PgPool Global Development Group
->>>>>>> fdfa148... Fix uninialized variables when all nodes are down.
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -252,6 +248,8 @@ from pool_read_message_length and recheck the pg_hba.conf settings.");
 	 */
 	for (;;)
 	{
+		char *message;
+
 		kind = pool_read_kind(cp);
 		if (kind < 0)
 		{
@@ -273,6 +271,10 @@ from pool_read_message_length and recheck the pg_hba.conf settings.");
 					break;
 
 				case 'N':
+					if (pool_extract_error_message(true, MASTER(cp), protoMajor, true, &message) == 1)
+					{
+						pool_log("pool_do_auth: notice message from backend:%s", message);
+					}
 					/* process notice message */
 					if (SimpleForwardToFrontend(kind, frontend, cp))
 						return -1;
@@ -281,6 +283,10 @@ from pool_read_message_length and recheck the pg_hba.conf settings.");
 
 					/* process error message */
 				case 'E':
+					if (pool_extract_error_message(true, MASTER(cp), protoMajor, true, &message) == 1)
+					{
+						pool_log("pool_do_auth: error message from backend:%s", message);
+					}
 					SimpleForwardToFrontend(kind, frontend, cp);
 					pool_flush(frontend);
 					return -1;
@@ -362,8 +368,8 @@ from pool_read_message_length and recheck the pg_hba.conf settings.");
 
 			cp->info[i].major = sp->major;
 			cp->info[i].minor = sp->minor;
-			strncpy(cp->info[i].database, sp->database, sizeof(cp->info[i].database) - 1);
-			strncpy(cp->info[i].user, sp->user, sizeof(cp->info[i].user) - 1);
+			strlcpy(cp->info[i].database, sp->database, sizeof(cp->info[i].database));
+			strlcpy(cp->info[i].user, sp->user, sizeof(cp->info[i].user));
 			cp->info[i].counter = 1;
 		}
 	}
@@ -1294,8 +1300,24 @@ signed char pool_read_kind(POOL_CONNECTION_POOL *cp)
 		{
 			if (kind != kind0)
 			{
+				char *message;
+
 				pool_error("pool_read_kind: kind does not match between master(%x) slot[%d] (%x)",
 						   kind0, i, kind);
+				if (kind0 == 'E')
+				{
+					if (pool_extract_error_message(false, MASTER(cp), MAJOR(cp), true, &message) == 1)
+					{
+						pool_log("pool_read_kind: error message from master backend:%s", message);
+					}
+				}
+				else if (kind == 'E')
+				{
+					if (pool_extract_error_message(false, CONNECTION(cp, i), MAJOR(cp), true, &message) == 1)
+					{
+						pool_log("pool_read_kind: error message from %d th backend:%s", i, message);
+					}
+				}
 				return -1;
 			}
 		}
